//**************************************************************************
// Implementation file:  DlgMemory.cpp

#include "stdafx.h"
#include "DlgMemory.h"

CDlgMemory::CDlgMemory()
{
	m_pDialog = NULL;
}

CDlgMemory::~CDlgMemory()
{
	ForgetState();
}

void CDlgMemory::Attach( CWnd* pDlg )
{
	m_pDialog = pDlg;
}

bool CDlgMemory::SaveRadioButtons( int nRadioID )
{
	ASSERT( m_pDialog ); // Don't forget to call Attach()
	CRadioMemory* pRadio = new CRadioMemory( nRadioID );
	pRadio->SaveState( m_pDialog );
	m_ControlList.AddTail( pRadio );
	return true;
}

bool CDlgMemory::SaveCheckBox( int nCheckBoxID )
{
	ASSERT( m_pDialog ); // Don't forget to call Attach()
	CCheckBoxMemory* pCheck = new CCheckBoxMemory( nCheckBoxID );
	pCheck->SaveState( m_pDialog );
	m_ControlList.AddTail( pCheck );
	return true;
}

bool CDlgMemory::SaveEditBoxText( int nEditBoxID )
{
	ASSERT( m_pDialog ); // Don't forget to call Attach()
	CEditBoxMemory* pEdit = new CEditBoxMemory( nEditBoxID );
	pEdit->SaveState( m_pDialog );
	m_ControlList.AddTail( pEdit );
	return true;
}

bool CDlgMemory::SaveSliderInfo( int nSliderID )
{
	ASSERT( m_pDialog ); // Don't forget to call Attach()
	CSliderMemory* pSlider = new CSliderMemory( nSliderID );
	pSlider->SaveState( m_pDialog );
	m_ControlList.AddTail( pSlider );
	return true;
}

bool CDlgMemory::SaveComboBoxSelection( int nComboBoxID )
{
	ASSERT( m_pDialog ); // Don't forget to call Attach()
	CComboBoxMemory* pCombo = new CComboBoxMemory( nComboBoxID );
	pCombo->SaveState( m_pDialog );
	m_ControlList.AddTail( pCombo );
	return true;
}

bool CDlgMemory::SaveListBoxSelection( int nListBoxID )
{
	ASSERT( m_pDialog ); // Don't forget to call Attach()
	CListBoxMemory* pListBox = new CListBoxMemory( nListBoxID );
	pListBox->SaveState( m_pDialog );
	m_ControlList.AddTail( pListBox );
	return true;
}

bool CDlgMemory::RestoreState()
{
	ASSERT( m_pDialog ); // Don't forget to call Attach()
	POSITION pos = m_ControlList.GetHeadPosition();
	while ( NULL != pos )
	{
		CControlMemory* pControlMemory = m_ControlList.GetNext( pos );
		ASSERT( pControlMemory );
		pControlMemory->RestoreState( m_pDialog );
	}
	return true;
}

void CDlgMemory::ForgetState()
{
	POSITION pos = m_ControlList.GetHeadPosition();
	while ( NULL != pos )
	{
		CControlMemory* pControlMemory = m_ControlList.GetNext( pos );
		ASSERT( pControlMemory );
		delete pControlMemory;
	}
	m_ControlList.RemoveAll();
}

/////////////////////////////////////////////////////////////////////////////

bool CRadioMemory::RestoreState( CWnd* pParent )
{
	// count the number of radio buttons
	int nCount = 0;

	CWnd* pRadio = pParent->GetDlgItem( m_nID );
	if ( !pRadio )
		return false;

	// main radio should have the group style set and needs to be a real radio button
	ASSERT( ::GetWindowLong( pRadio->GetSafeHwnd(), GWL_STYLE ) & WS_GROUP );
	ASSERT( pRadio->SendMessage( WM_GETDLGCODE, 0, 0L ) & DLGC_RADIOBUTTON );

	// for each radio in the group...
	do
	{
		// get out if this isn't a radio button - set tab order properly, please
		if ( !( pRadio->SendMessage( WM_GETDLGCODE, 0, 0L ) & DLGC_RADIOBUTTON ) )
			break;

		// restore the state
		pRadio->SendMessage( BM_SETCHECK, ( m_nState == nCount ), 0L );

		// next radio button
		pRadio = pRadio->GetNextWindow();
		nCount++;
	}
	// while the radio button is valid and it doesn't have the group style.
	while ( NULL != pRadio && !( ::GetWindowLong( pRadio->GetSafeHwnd(), GWL_STYLE) & WS_GROUP ) );

	return true;
}

bool CRadioMemory::SaveState( CWnd* pParent )
{
	int nCount = 0;

	CWnd* pRadio = pParent->GetDlgItem( m_nID );
	if ( !pRadio )
		return false;

	// main radio should have the group style set and needs to be a real radio button
	ASSERT( ::GetWindowLong( pRadio->GetSafeHwnd(), GWL_STYLE ) & WS_GROUP );
	ASSERT( pRadio->SendMessage( WM_GETDLGCODE, 0, 0L ) & DLGC_RADIOBUTTON );

	// count the number of radio buttons
	do
	{
		if ( !( pRadio->SendMessage( WM_GETDLGCODE, 0, 0L ) & DLGC_RADIOBUTTON ) )
			break;

		if ( 0 != pRadio->SendMessage( BM_GETCHECK, 0, 0L ) )
			m_nState = nCount;

		pRadio = pRadio->GetNextWindow();
		nCount++;
	}
	while ( NULL != pRadio && 
		!( ::GetWindowLong( pRadio->GetSafeHwnd(), GWL_STYLE) & WS_GROUP ) );

	return true;
}

/////////////////////////////////////////////////////////////////////////////

bool CCheckBoxMemory::RestoreState( CWnd* pParent )
{
	CWnd* pCheck = pParent->GetDlgItem( m_nID );
	if ( !pCheck )
		return false;

	if ( m_nState < 0 || m_nState > 2 )
		return false;

	pCheck->SendMessage( BM_SETCHECK, (WPARAM)m_nState, 0L );

	return true;
}

bool CCheckBoxMemory::SaveState( CWnd* pParent )
{
	CWnd* pCheck = pParent->GetDlgItem( m_nID );
	if ( !pCheck )
		return false;

	ASSERT( m_nState >= 0 || m_nState <= 2 );

	m_nState = pCheck->SendMessage( BM_GETCHECK, 0, 0L );

	return true;
}

/////////////////////////////////////////////////////////////////////////////

bool CEditBoxMemory::RestoreState( CWnd* pParent )
{
	CWnd* pEdit = pParent->GetDlgItem( m_nID );
	if ( !pEdit )
		return false;

	pEdit->SetWindowText( m_sState );

	return true;
}

bool CEditBoxMemory::SaveState( CWnd* pParent )
{
	CWnd* pEdit = pParent->GetDlgItem( m_nID );
	if ( !pEdit )
		return false;

	pEdit->GetWindowText( m_sState );

	return true;
}

/////////////////////////////////////////////////////////////////////////////

bool CSliderMemory::RestoreState( CWnd* pParent )
{
	CSliderCtrl* pSlider = (CSliderCtrl*)pParent->GetDlgItem( m_nID );
	if ( !pSlider )
		return false;

	pSlider->SetPos( m_nPos );

	return true;
}

bool CSliderMemory::SaveState( CWnd* pParent )
{
	CSliderCtrl* pSlider = (CSliderCtrl*)pParent->GetDlgItem( m_nID );
	if ( !pSlider )
		return false;

	int nLower, nUpper;
	pSlider->GetRange( nLower, nUpper );
	m_nPos = pSlider->GetPos();

	ASSERT( nLower <= nUpper );
	ASSERT( m_nPos >= nLower && m_nPos <= nUpper );

	return true;
}

/////////////////////////////////////////////////////////////////////////////

bool CComboBoxMemory::RestoreState( CWnd* pParent )
{
	CWnd* pCombo = pParent->GetDlgItem( m_nID );
	if ( !pCombo )
		return false;
	
	pCombo->SendMessage( CB_SETCURSEL, (WPARAM)m_nSelection, 0L);

	return true;
}

bool CComboBoxMemory::SaveState( CWnd* pParent )
{
	CWnd* pCombo = pParent->GetDlgItem( m_nID );
	if ( !pCombo )
		return false;

	m_nSelection = (int)pCombo->SendMessage( CB_GETCURSEL, 0, 0L );

	return true;
}

/////////////////////////////////////////////////////////////////////////////

bool CListBoxMemory::RestoreState( CWnd* pParent )
{
	CWnd* pList = pParent->GetDlgItem( m_nID );
	if ( !pList )
		return false;

	// if it's a multiselect list box...
	if ( ::GetWindowLong( pList->GetSafeHwnd(), GWL_STYLE ) & LBS_MULTIPLESEL )
	{
		for ( int i = 0; i < m_nCount; i++ )
			pList->SendMessage( LB_SETSEL, (WPARAM)TRUE, m_pSelection[ i ] );
	}
	// single select list box
	else
	{
		if ( NULL == m_pSelection )
			return false;
		pList->SendMessage( LB_SETCURSEL, (WPARAM)*m_pSelection, 0L );
	}

	return true;
}

bool CListBoxMemory::SaveState( CWnd* pParent )
{
	CWnd* pList = pParent->GetDlgItem( m_nID );
	if ( !pList )
		return false;

	// if it's a multiselect list box...
	if ( ::GetWindowLong( pList->GetSafeHwnd(), GWL_STYLE ) & LBS_MULTIPLESEL )
	{
		// how many items are selected?
		m_nCount = pList->SendMessage( LB_GETSELCOUNT, 0, 0L );
		// allocate an array of ints based on the count
		m_pSelection = new int[ m_nCount ];
		// get the indexes of the selected items, pass the array's pointer
		pList->SendMessage( LB_GETSELITEMS, (WPARAM)m_nCount, (LPARAM)m_pSelection );
	}
	// single select list box
	else
	{
		UINT nSelection = (UINT)pList->SendMessage( LB_GETCURSEL, 0, 0L );
		m_nCount = 1;
		if ( NULL == m_pSelection )
			m_pSelection = new int[ m_nCount ];

		*m_pSelection = nSelection;
	}

	return true;
}
